# -*- coding: utf-8 -*-

import os
import platform
import re
import shutil
import subprocess
import time
from subprocess import CalledProcessError

from loguru import logger


def run_command(command):
    system = platform.system()
    if system == 'Linux':
        subprocess.run(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True)
    elif system == "Darwin":
        subprocess.run("source ~/.bash_profile && " + str(command), shell=True, stdout=subprocess.PIPE,
                       stderr=subprocess.PIPE, check=True)
    elif system == 'Windows':
        windows_command = ['cmd', '/c']
        command_detail_list = re.split(r"[ ]+", command)
        windows_command.extend(command_detail_list)
        subprocess.run(windows_command, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True)
    else:
        raise RuntimeError


class AllureTools(object):
    """
    Allure测试报告操作类
    """

    def __init__(self, result_path, report_path, resource_path=None):
        """
        初始化Allure
        :param result_path: Allure结果文件目录
        :param report_path: Allure报告文件目录
        :param resource_path: Allure外部资源文件目录（即environment.properties及categories.properties所在目录）
        """
        self._ALLURE_RESULT = result_path
        self._ALLURE_REPORT = report_path
        self._RESOURCE_PATH = resource_path

    def check_result(self, check_count=3):
        """
        检查Allure测试结果json文件是否生成。
        :param check_count: 检测次数（默认为3）。
        :return:
        """
        check_result = None
        for count in range(check_count):
            if os.listdir(self._ALLURE_RESULT) == list() and count < check_count:
                if count == check_count - 1:
                    logger.warning("仍未检测到Allure测试结果json文件，已达检测次数上限({})，停止检测。".format(check_count))
                    check_result = False
                    break
                logger.warning("未检测到Allure测试结果json文件，可能正在生成......")
                time.sleep(2)
                continue
            if os.listdir(self._ALLURE_RESULT) != list():
                logger.info("[Success]：已检测到Allure测试结果json文件.")
                check_result = True
                break
        return check_result

    def generate_report(self):
        """
        根据json结果文件自动生成Allure测试报告。
        """
        try:
            if self.check_result() is True:
                command = "allure generate {0} -o {1} --clean".format(self._ALLURE_RESULT, self._ALLURE_REPORT)
                time.sleep(1)
                logger.info('开始执行Allure测试报告生成命令："{}"'.format(command))
                run_command(command)
                logger.info("[Done]：已经成功生成Allure测试报告.")
            else:
                logger.warning("[Warning]：由于未检测到Allure测试结果json文件，停止生成Allure测试报告！")
        except CalledProcessError:
            logger.exception("[Exception]：Allure测试报告生成命令执行失败！")
        except Exception:
            logger.exception("[Exception]：生成Allure测试报告过程中发生异常，请检查！")

    def clear_result(self):
        """
        清空Allure测试结果json文件。
        """
        try:
            if os.listdir(self._ALLURE_RESULT) != list():
                time.sleep(1)
                shutil.rmtree(self._ALLURE_RESULT)
                os.mkdir(self._ALLURE_RESULT)
                logger.info("[Success]：已经成功清空Allure历史测试结果.")
            else:
                logger.info("当前暂无Allure历史测试结果，无需清除操作！")
        except Exception:
            logger.exception("[Exception]：清空Allure历史测试结果过程中发生异常，请检查！")

    def sync_history(self):
        """
        追加Allure历史追溯信息
        :return:
        """
        ALLURE_RESULT_HISTORY = os.path.join(self._ALLURE_RESULT, "history")
        ALLURE_REPORT_HISTORY = os.path.join(self._ALLURE_REPORT, "history")
        try:
            if os.path.exists(ALLURE_RESULT_HISTORY):
                raise FileExistsError
            if os.path.exists(ALLURE_REPORT_HISTORY):
                time.sleep(1)
                shutil.copytree(ALLURE_REPORT_HISTORY, ALLURE_RESULT_HISTORY)
                logger.info("[Success]：已经成功同步Allure历史追溯信息.")
            else:
                logger.warning('[WARNING]：Allure历史追溯信息"{}"当前并不存在，无法完成同步！'.format(ALLURE_REPORT_HISTORY))
        except FileExistsError:
            logger.exception('[Exception]：已同步Allure历史追溯信息至"{}"，无需再次同步！'.format(ALLURE_RESULT_HISTORY))
        except Exception:
            logger.exception("[Exception]：同步Allure历史追溯信息过程中发生异常，请检查！")

    def sync_environment(self):
        """
        同步Allure环境信息文件
        :return:
        """
        if self._RESOURCE_PATH is None:
            pass
        else:
            ENVIRONMENT_INFO = os.path.join(self._RESOURCE_PATH, "environment.properties")
            try:
                if os.path.exists(ENVIRONMENT_INFO):
                    time.sleep(1)
                    shutil.copyfile(ENVIRONMENT_INFO, os.path.join(self._ALLURE_RESULT, "environment.properties"))
                    logger.info("[Success]：已经成功同步Allure环境信息文件.")
                else:
                    raise FileNotFoundError
            except FileNotFoundError:
                logger.exception('[Exception]：Allure环境信息文件"{}"并不存在，无法同步，请检查！'.format(ENVIRONMENT_INFO))
            except Exception:
                logger.exception("[Exception]：同步Allure环境信息文件过程中发生异常，请检查！")

    def sync_categories(self):
        """
        同步Allure测试分类文件
        :return:
        """
        if self._RESOURCE_PATH is None:
            pass
        else:
            CATEGORIES_INFO = os.path.join(self._RESOURCE_PATH, "categories.json")
            try:
                if os.path.exists(CATEGORIES_INFO):
                    time.sleep(1)
                    shutil.copyfile(CATEGORIES_INFO, os.path.join(self._ALLURE_RESULT, "categories.json"))
                    logger.exception("[Success]：已经成功同步Allure测试分类文件.")
                else:
                    raise FileNotFoundError
            except FileNotFoundError:
                logger.exception('[Exception]：Allure测试分类文件"{}"并不存在，无法同步，请检查！'.format(CATEGORIES_INFO))
            except Exception:
                logger.exception("[Exception]：同步Allure测试分类文件过程中发生异常，请检查！")

    def initial_allure(self):
        """
        初始化Allure
        :return:
        """
        try:
            logger.info('[Initial]：开始初始化Allure......')
            self.clear_result()
            if os.path.exists(self._ALLURE_REPORT):
                self.sync_history()
            self.sync_environment()
            self.sync_categories()
            logger.info("[Done]：已经成功初始化Allure.")
        except Exception:
            logger.exception("[Exception]：初始化Allure过程中发生异常，请检查！")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pypanzer', 'pypanzer.tools']

package_data = \
{'': ['*']}

install_requires = \
['SQLAlchemy>=1.3.9,<2.0.0',
 'emoji>=1.2.0,<2.0.0',
 'loguru>=0.4.1,<0.5.0',
 'pandas==1.1.5',
 'pymongo>=3.9.0,<4.0.0',
 'pymysql>=0.9.3,<0.10.0',
 'redis>=3.5.3,<4.0.0',
 'requests>=2.22.0,<3.0.0',
 'selenium>=3.141.0,<4.0.0',
 'xmltodict>=0.12.0,<0.13.0']

setup_kwargs = {
    'name': 'pypanzer',
    'version': '1.0.3',
    'description': 'panzer tools for python language',
    'long_description': None,
    'author': '于中华',
    'author_email': '583512498@qq.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-

import calendar
import datetime
import json
import os
import platform
import random
import re
import socket
import string
import subprocess
import time
from collections import OrderedDict

import emoji
import requests
import xmltodict
from loguru import logger
from selenium import webdriver
from selenium.webdriver.chrome.options import Options


def is_number(s):
    """
    判断标识符是否为数字
    :param s:
    :return:
    """
    try:
        float(s)
        return True
    except ValueError:
        pass
    try:
        import unicodedata
        unicodedata.numeric(s)
        return True
    except (TypeError, ValueError):
        pass
    return False


def random_str(length):
    """
    生成指定长度的随机字符串.
    :param length: 指定长度.
    :return:
    """
    str_list = random.sample(string.digits + string.ascii_letters, length)
    random_str = ''.join(str_list).lower()
    return random_str


def calc_round(n, m=0):
    """
    四舍五入精度
    :param n: 浮点数
    :param m: 进位数
    :return:
    """
    if m == 0:
        return int(n + 0.5)
    temp_list = str(n).split('.')
    if temp_list.__len__() != 1:
        temp = temp_list[1][:m]
        n = str(int(n * 10 ** m + 0.5))
        n = n.rjust(temp.__len__(), '0')
    else:
        n = str(int(n * 10 ** m + 0.5))
    m *= -1
    n = '{}.{}'.format(n[:m], n[m:])
    return float(n)


def hum_convert(value):
    """
    根据字节自适应转换为B、KB、MB、GB等
    :param value:
    :return:
    """
    units = ["B", "KB", "MB", "GB", "TB", "PB"]
    size = 1024.0
    for i in range(len(units)):
        if (value / size) < 1:
            return "%.2f%s" % (value, units[i])
        value = value / size


def time_spent(func):
    """
    运行耗时装饰器.
    :return:
    """

    def wrapper(*args, **kw):
        start_time = time.time()
        result = func(*args, **kw)
        logger.info('被装饰方法{}总共运行耗时{}秒.'.format(func.__name__, round(time.time() - start_time, 2)))
        return result

    return wrapper


def trans_timestamp(timeNum):
    """
    13位时间戳转换为指定时间格式.
    :param timeNum: 13 format timestamp string, such as 1573120029224
    :return: output such as 2019-11-07 17:47:09
    """
    timeStamp = float(timeNum / 1000)
    timeArray = time.localtime(timeStamp)
    otherStyleTime = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
    return otherStyleTime


def date_timedelta(date_str):
    """
    获取指定日期的星期、季度、月份、年份分布
    :param date_str: 字符串格式 '2020-08-21'
    :return:
    """
    result = {}
    struct_date = datetime.datetime.strptime(date_str, "%Y-%m-%d")
    # 获取当前月第一天和最后一天
    _, monthRange = calendar.monthrange(struct_date.year, struct_date.month)
    firstDay_month = datetime.date(year=struct_date.year, month=struct_date.month, day=1).strftime("%Y-%m-%d")
    lastDay_month = datetime.date(year=struct_date.year, month=struct_date.month, day=monthRange).strftime("%Y-%m-%d")

    # 获取当前年份第一天和最后一天
    _, last_month_of_days = calendar.monthrange(struct_date.year, 12)
    first_day_year = datetime.date(year=struct_date.year, month=1, day=1).strftime("%Y-%m-%d")
    last_day_year = datetime.date(year=struct_date.year, month=12, day=last_month_of_days).strftime("%Y-%m-%d")

    # 获取当前周第一天和最后一天
    this_week_start = (struct_date - datetime.timedelta(days=struct_date.weekday())).strftime("%Y-%m-%d")
    this_week_end = (struct_date + datetime.timedelta(days=6 - struct_date.weekday())).strftime("%Y-%m-%d")

    # 获取本季度第一天和最后一天
    month = struct_date.month
    if month in [1, 2, 3]:
        this_quarter_start = datetime.date(struct_date.year, 1, 1).strftime("%Y-%m-%d")
        this_quarter_end = (datetime.date(struct_date.year, 4, 1) - datetime.timedelta(days=1)).strftime("%Y-%m-%d")
    elif month in [4, 5, 6]:
        this_quarter_start = datetime.date(struct_date.year, 4, 1).strftime("%Y-%m-%d")
        this_quarter_end = (datetime.date(struct_date.year, 7, 1) - datetime.timedelta(days=1)).strftime("%Y-%m-%d")
    elif month in [7, 8, 9]:
        this_quarter_start = datetime.date(struct_date.year, 7, 1).strftime("%Y-%m-%d")
        this_quarter_end = (datetime.date(struct_date.year, 10, 1) - datetime.timedelta(days=1)).strftime("%Y-%m-%d")
    else:
        this_quarter_start = datetime.date(struct_date.year, 10, 1).strftime("%Y-%m-%d")
        this_quarter_end = (datetime.date(struct_date.year + 1, 1, 1) - datetime.timedelta(days=1)).strftime("%Y-%m-%d")

    result.update({
        'year': {'first_day': first_day_year, 'last_day': last_day_year},
        'month': {'first_day': firstDay_month, 'last_day': lastDay_month},
        'quarter': {'first_day': this_quarter_start, 'last_day': this_quarter_end},
        'week': {'first_day': this_week_start, 'last_day': this_week_end}, })
    return result


def run_command(command):
    """
    执行终端命令.
    :param command: 字符串形式命令(根据系统识别命令方式)
    :return:
    """
    system = platform.system()
    if system == 'Linux':
        subprocess.run(command, shell=True, stdout=subprocess.PIPE,
                       stderr=subprocess.PIPE, check=True)
    elif system == "Darwin":
        subprocess.run("source ~/.bash_profile && " + str(command), shell=True, stdout=subprocess.PIPE,
                       stderr=subprocess.PIPE, check=True)
    elif system == 'Windows':
        windows_command = ['cmd', '/c']
        command_detail_list = re.split(r"[ ]+", command)
        windows_command.extend(command_detail_list)
        subprocess.run(windows_command, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True)
    else:
        raise RuntimeError


def pipe_command(command):
    """
    执行终端命令.
    :param command: 字符串形式命令(根据系统识别命令方式)
    :return:
    """
    system = platform.system()
    if system == 'Linux':
        subprocess.Popen(command, shell=True, stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    elif system == "Darwin":
        subprocess.Popen("source ~/.bash_profile && " + str(command), shell=True, stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    elif system == 'Windows':
        windows_command = ['cmd', '/c']
        command_detail_list = re.split(r"[ ]+", command)
        windows_command.extend(command_detail_list)
        subprocess.Popen(windows_command, shell=False, creationflags=subprocess.CREATE_NEW_CONSOLE)
    else:
        raise RuntimeError


def unicode_to_normal(origin_str):
    """
    将中文unicode字符(如"\u6d4b")转换为正常中文
    :param origin_str: 中文unicode字符
    :return:
    """
    result_str = str(origin_str).encode("utf-8").decode("unicode-escape")
    return result_str


def param_to_dict(request_param):
    """
    转换请求参数为字典形式, 如a=1&b=2转换为{"a":1, "b":2}
    :param request_param: 请求参数
    :return: dict
    """
    if str(request_param).strip() == "":
        return {}
    param_zip_list = [tuple([i.strip() for i in re.split(r"[=]+", param.strip())]) for param in
                      re.split(r"[&]+", request_param)]
    param_dict = dict(param_zip_list)
    return param_dict


def body_to_dict(consumes, request_body):
    """
    根据consumes将请求体统一转换为字典形式
    :param consumes: 请求类型
    :param request_body: 请求体
    :return:
    """
    consume = str(consumes).lower().split(r';')[0].strip()
    if consume not in ["multipart/form-data", "application/x-www-form-urlencoded", "application/xml",
                       "application/json", "*/*"]:
        return request_body
    if str(request_body).strip() == "":
        return {}
    if consume == "multipart/form-data":
        return request_body
    elif consume == "application/x-www-form-urlencoded":
        return param_to_dict(request_body)
    elif consume == "application/xml":
        return json.loads(xml_to_json(request_body))
    elif consume == "application/json" or consume == "*/*":
        return json.loads(request_body)
    else:
        return request_body


def xml_to_json(xml_str):
    """
    XML字符串转换为JSON字符串
    :param xml_str: XML字符串
    :return:
    """
    if str(xml_str).strip() == "":
        return "{}"
    xml_str = re.split(pattern=r'<\?[\s\S]*\?>', string=str(xml_str).strip(), maxsplit=1)[-1]
    xml_parse = xmltodict.parse(xml_str)
    json_str = json.dumps(xml_parse, indent=1)
    return json_str


def json_to_xml(json_dict):
    """
    JSON字典转换为XML字符串
    :param json_dict: json字典
    :return:
    """
    xml_str = xmltodict.unparse(json_dict, pretty=1)
    return xml_str


def emoji_to_str(origin):
    """
    将Emoji表情转换为字符窜
    :param origin:
    :return:
    """
    if isinstance(origin, str):
        return emoji.demojize(origin)
    elif isinstance(origin, list):
        return list(map(lambda x: emoji_to_str(x), origin))
    elif isinstance(origin, tuple):
        return tuple(map(lambda x: emoji_to_str(x), origin))
    elif isinstance(origin, set):
        return set(map(lambda x: emoji_to_str(x), origin))
    elif isinstance(origin, dict):
        keys = list(map(lambda x: emoji_to_str(x), list(origin.keys())))
        values = list(map(lambda x: emoji_to_str(x), list(origin.values())))
        return dict(zip(keys, values))
    else:
        return origin


def capture_image(width, height, url, sleep, pic):
    """
    通过Selenium进行截屏并保存
    :param width: 浏览器宽度
    :param height: 浏览器高度
    :param url: 待浏览网址
    :param sleep: 加载等待时间
    :param pic: 截屏存取路径
    :return:
    """
    try:
        logger.info("[ScreenShot]：开始截屏，请稍等...")
        chrome_options = Options()
        chrome_options.add_argument('--headless')
        driver = webdriver.Chrome(options=chrome_options)
        driver.set_window_size(width=width, height=height)
        driver.get(url=url)
        time.sleep(sleep)
        driver.save_screenshot(pic)
        driver.close()
        driver.quit()
        logger.info("[Success]：截屏成功，存取路径为{}".format(pic))
        return pic
    except Exception:
        logger.exception("[Failure]：截屏失败，请检查并重试！！！")


def sm_ms(pic):
    """
    上传图片到sm.ms图床
    :param pic: 图片本地路径
    :return:
    """
    # 需要登录sm.ms去获取自己的API Token
    headers = {'Authorization': 'KdaYAzW2XW6WzLqivXT7UNlSj1mW1fyD'}
    history_url = "https://sm.ms/api/v2/upload_history"
    delete_url = "https://sm.ms/api/v2/delete/<hashid>"
    upload_url = 'https://sm.ms/api/v2/upload'

    def upload(pic):
        files = {'smfile': open(pic, 'rb')}
        upload_res_json = requests.post(upload_url, files=files, headers=headers).json()
        if upload_res_json["success"] is True:
            result_dict = dict(view_url=upload_res_json["data"]["url"], remove_url=upload_res_json["data"]["delete"])
            logger.info("图片上传sm.ms成功，其在线访问信息为：{}".format(result_dict))
            return result_dict.get("view_url")
        else:
            logger.error("图片上传sm.ms失败！其返回错误信息为：{}".format(upload_res_json["message"]))

    history_res_json = requests.get(url=history_url, headers=headers).json()
    history_pic_data = history_res_json.get("data")
    if history_pic_data == []:
        logger.info("当前sm.ms不存在任何图片信息，开始上传图片...")
        return upload(pic=pic)
    else:
        history_pic_infos = [dict(filename=pic_info["filename"], hashid=pic_info["hash"]) for pic_info in
                             history_pic_data]
        history_pic_names = [pic_info.get("filename") for pic_info in history_pic_infos]
        now_pic_name = os.path.split(pic)[1]
        if now_pic_name in history_pic_names:
            logger.warning("当前sm.ms已存在图片{}，开始自动删除...".format(now_pic_name))
            now_pic_hashid = \
                [pic_info["hashid"] for pic_info in history_pic_infos if pic_info["filename"] == now_pic_name][0]
            delete_res_json = requests.get(url=delete_url.replace("<hashid>", now_pic_hashid)).json()
            if delete_res_json["success"] is True:
                logger.info("sm.ms图片{}删除成功，开始重新上传图片...".format(now_pic_name))
                return upload(pic=pic)
            else:
                logger.error("sm.ms图片{}删除失败，失败信息为：{}".format(now_pic_name, delete_res_json["message"]))
                logger.info("继续尝试上传图片...")
                return upload(pic=pic)
        else:
            logger.info("当前sm.ms不存在图片{}，开始上传图片...".format(now_pic_name))
            return upload(pic=pic)


def face_bed(pic, alias):
    """
    上传图片到遇见图床
    :param pic: 图片存取地址
    :param alias: 图片名称别名
    :return:
    """
    url = "https://www.hualigs.cn/api/upload"
    payload = {'apiType': 'bilibili',
               'folder': 1082,
               'field': alias,
               'token': 'e274d5dbb94e919ab26cc5fca22692c3'}  # 需要登录hualigs.cn去获取自己的API Token
    files = [
        ('image', open(pic, 'rb'))
    ]
    res_json = requests.post(url, data=payload, files=files).json()
    if res_json["msg"] == "success":
        cdn_url = res_json["data"]["url"]["bilibili"]
        logger.info("图片上传成功，CDN地址为：{}".format(cdn_url))
        return cdn_url
    else:
        logger.error("图片上传失败！其返回错误信息为：{}".format(res_json))


def get_alphabet(number):
    """
    将数字转换为大写字母
    :param number: 大于等于0的整数
    :return: 大写字母，如A或CW等
    """
    factor, moder = divmod(number, 26)
    modChar = chr(moder + 65)
    if factor != 0:
        modChar = get_alphabet(factor - 1) + modChar
    return modChar


def get_ip():
    """
    获取本机IP地址
    :return:
    """
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
    return ip


def value_by_type(dest_value, test_value):
    """
    转换测试值为目标值类型的数据
    :param dest_value: 目标值
    :param test_value: 测试值
    :return:
    """
    if isinstance(test_value, type(dest_value)) or dest_value is None:
        return test_value
    else:
        type_str = str(type(dest_value)).split(r"'")[1]
        test_value = eval(type_str)(test_value)
        return test_value


def func_by_eval(func_expr):
    """
    eval方式解析函数表达式并求值
    :param func_expr: 函数表达式，如"${func(a, b)}"
    :return:
    """
    if re.match(r'^\$\{.*\}$', str(func_expr)):
        expr = func_expr[2:-1]
        return eval(expr)
    else:
        return func_expr


def force_to_json(origin):
    """
    json字串转换为字典
    :param origin: 输入字符串
    :return:
    """
    if re.match(r"(^\{[\s\S]*\}$)|(^\[\{[\s\S]*\}\]$|\[\])", str(origin)):
        return json.loads(emoji.demojize(json.dumps(json.loads(emoji.demojize(str(origin))), ensure_ascii=False)))
    else:
        if isinstance(origin, str):
            origin = emoji.demojize(origin)
            if re.search(pattern=r'<\?[\s\S]*\?>', string=str(origin).strip()) is not None:
                return {"qa": json.loads(xml_to_json(origin))}
            else:
                return {"qa": origin}
        else:
            return {"qa": origin}


def sortDict(pyload, reverse=False):
    """
    字典深层递归排序
    :param pyload: 待排序字典
    :param reverse: 排序规则(默认升序)
    :return: 排序后OrderedDict字典
    """
    item = 'p.items()'
    if type(pyload).__name__ == 'list':
        p = sorted(pyload, reverse=reverse)
        item = 'enumerate(p)'
    if type(pyload).__name__ == 'dict':
        p = OrderedDict(sorted(pyload.items(), key=lambda a: a[0], reverse=reverse))
    for k, v in eval(item):
        if type(v).__name__ == 'list':
            if not v or (v is None):
                p.pop(k)
            else:
                p[k] = list(sortDict(sorted(v, reverse=reverse), reverse=reverse))
        elif type(v).__name__ == 'dict':
            if not v or (v is None):
                p.pop(k)
            else:
                p[k] = dict(sortDict(v, reverse=reverse))
            return p
        else:
            if v is None:
                p.pop(k)
    return p



# -*- coding: utf-8 -*-

import os
import sys

import pandas as pd
import redis
from loguru import logger
from pymongo import MongoClient
from redis.sentinel import Sentinel
from sqlalchemy import create_engine


class MysqlTools(object):
    """
    MySQL数据库工具类
    """

    def __init__(self, host, port, username, password, dbname):
        """
        指定所处环境及数据库名称
        :param host: 主机地址
        :param port: 主机端口
        :param username: 用户
        :param password: 密码
        """
        self.mysql_engine = create_engine(
            "mysql+pymysql://{}:{}@{}:{}/{}?charset=utf8".format(username, password, host, port, dbname))

    @property
    def mysql(self):
        """
        返回MySql会话
        :return:
        """
        return self.mysql_engine

    def update_string(self, update_sql):
        """
        更新语句操作
        :param update_sql: 更新SQL
        :return:
        """
        update_result = self.mysql_engine.execute(update_sql)
        return update_result.rowcount

    def delete_string(self, delete_sql):
        """
        删除语句操作
        :param delete_sql: 删除SQL
        :return:
        """
        delete_result = self.mysql_engine.execute(delete_sql)
        return delete_result.rowcount

    def query_string(self, script_str):
        """
        通过sqlalchemy并使用engine方式执行脚本(字符串)并返回结果
        :param script_str: 脚本字符串
        :return:
        """
        try:
            result = self.mysql_engine.execute(script_str).fetchall()
            return result
        except Exception:
            logger.exception("[Exception]：执行MySQL脚本过程中发生异常，请检查！~ 脚本为：【{}】".format(script_str))

    def query_file(self, script_file):
        """
        通过sqlalchemy并使用engine方式执行脚本(文本文件)并返回结果
        :param script_file: 脚本文件
        :return:
        """
        script_string = None
        try:
            if os.path.exists(script_file):
                with open(script_file, mode=r'r', encoding='utf-8') as script:
                    script_string = script.read()
                    result = self.mysql_engine.execute(script_string).fetchall()
                    return result
            else:
                raise FileNotFoundError
        except FileNotFoundError:
            logger.exception('[Exception]：MySQL脚本文件"{}"当前并不存在'.format(script_file))
        except Exception:
            logger.exception("[Exception]：执行MySQL脚本过程中发生异常，请检查！~ 脚本为：【{}】".format(script_string))

    def query_string_df(self, script_str, index_col=None, columns=None, chunksize=None):
        """
        通过sqlalchemy并使用read_sql方式执行脚本(字符串)并返回结果
        :param script_str: 脚本字符串
        :param index_col: 将指定列作为索引
        :param columns: 指定获取列(read_sql_table时候可用)
        :param chunksize: 缓存区大小
        :return:
        """
        try:
            result = pd.read_sql(script_str, con=self.mysql_engine, index_col=index_col, columns=columns,
                                 chunksize=chunksize)
            return result
        except Exception:
            logger.exception("[Exception]：执行MySQL脚本过程中发生异常，请检查！~ 脚本为：【{}】".format(script_str))

    def query_file_df(self, script_file, index_col=None, chunksize=None):
        """
        通过sqlalchemy并使用read_sql_query方式执行脚本文件并返回结果
        :param script_file: 脚本文件
        :param index_col: 将指定列作为索引
        :param chunksize: 缓存区大小
        :return:
        """
        script_string = None
        try:
            if os.path.exists(script_file):
                with open(script_file, mode=r'r', encoding='utf-8') as script:
                    script_string = script.read()
                    result = pd.read_sql_query(script_string, con=self.mysql_engine, index_col=index_col,
                                               chunksize=chunksize)
                    return result
            else:
                raise FileNotFoundError
        except FileNotFoundError:
            logger.exception('[Exception]：MySQL脚本文件"{}"当前并不存在'.format(script_file))
        except Exception:
            logger.exception("[Exception]：执行MySQL脚本过程中发生异常，请检查！~ 脚本为：【{}】".format(script_string))


class MongoTools(object):
    """
    MongoDB数据库工具类
    """

    def __init__(self, host, port, dbname, auth_dict=None):
        """
        指定所处环境及数据库名称
        :param host: 主机地址
        :param port: 主机端口
        :param dbname: 数据库名称
        :param auth_dict: 授权字典，样例格式为{"auth_db":"admin", "auth_username":"root", "auth_password":"123456"}
        """
        self.mongodb = MongoClient(host=host, port=port)
        if auth_dict is not None:
            if isinstance(auth_dict, dict):
                auth_db = auth_dict.get("auth_db")
                auth_username = auth_dict.get("auth_username")
                auth_password = str(auth_dict.get("auth_password"))
                auth = eval("self.mongodb.{}".format(auth_db))
                auth.authenticate(auth_username, auth_password)
                self.db_client = eval("self.mongodb.{}".format(dbname))
            else:
                logger.warning("auth_dict授权信息必须为字典形式，请检查！")
                sys.exit(1)

    def db(self, dbname=None):
        """
        获取MongoDB数据库会话
        :param dbname: 数据库名称
        :return:
        """
        if dbname is None:
            return self.DbClient(db_client=self.db_client)
        else:
            db_client = eval("self.mongodb.{}".format(dbname))
            return self.DbClient(db_client=db_client)

    @property
    def mongo(self):
        """
        获取MongoDB实例
        :return:
        """
        return self.mongodb

    class DbClient:
        """
        MongoDB数据库操作
        """

        def __init__(self, db_client):
            """
            初始化MongoDB数据库实例
            :param db_client: MongoDB数据库实例
            """
            self.db_client = db_client

        @property
        def client(self):
            """
            获取MongoDB数据库实例
            :return:
            """
            return self.db_client

        def drop(self, tbname):
            """
            删除指定表
            :param tbname: MongoDB表名
            :return:
            """
            tb = eval("self.db_client.{}".format(tbname))
            drop = tb.drop()
            return drop

        def insert_one(self, tbname, insert_json):
            """
            单条数据插入
            :param tbname: MongoDB表名
            :param insert_json: 单条插入json, 最好指定_id否则可能重复插入
            :return: 插入数据id
            """
            tb = eval("self.db_client.{}".format(tbname))
            insert = tb.insert_one(insert_json)
            return insert.inserted_id

        def insert_many(self, tbname, insert_json_list):
            """
            批量数据插入
            :param tbname: MongoDB表名
            :param insert_json_list: 批量插入json列表，最好指定_id否则可能重复插入
            :return: 插入数据id列表
            """
            tb = eval("self.db_client.{}".format(tbname))
            insert = tb.insert_many(insert_json_list)
            return insert.inserted_ids

        def delete_one(self, tbname, delete_json):
            """
            删除匹配到的第一条数据
            :param tbname: MongoDB表名
            :param delete_json: 删除数据json, _id必须用ObjectId("5ff2c39364efd65eed78597e")方式
            :return: 删除数据id
            """
            tb = eval("self.db_client.{}".format(tbname))
            delete = tb.delete_one(delete_json)
            return delete.deleted_count

        def delete_many(self, tbname, delete_json):
            """
            删除多条数据
            :param tbname: MongoDB表名
            :param delete_json: 删除数据json，若为{}则删除集合全部数据, _id必须用ObjectId("5ff2c39364efd65eed78597e")方式
            :return: 删除数据id
            """
            tb = eval("self.db_client.{}".format(tbname))
            delete = tb.delete_many(delete_json)
            return delete.deleted_count

        def update_one(self, tbname, filter_json, update_json, upsert=False):
            """
            更新匹配到的第一条数据
            :param tbname: MongoDB表名
            :param filter_json: 过滤json, _id必须用ObjectId("5ff2c39364efd65eed78597e")方式
            :param update_json: 更新json
            :param upsert: 更新插入开关
            :return:
            """
            tb = eval("self.db_client.{}".format(tbname))
            update = tb.update_one(filter=filter_json, update=update_json, upsert=upsert)
            return update.modified_count

        def update_many(self, tbname, filter_json, update_json, upsert=False):
            """
            更新多条数据
            :param tbname: MongoDB表名
            :param filter_json: 过滤json, _id必须用ObjectId("5ff2c39364efd65eed78597e")方式
            :param update_json: 更新json
            :param upsert: 更新插入开关
            :return:
            """
            tb = eval("self.db_client.{}".format(tbname))
            update = tb.update_many(filter=filter_json, update=update_json, upsert=upsert)
            return update.modified_count

        def find_one(self, tbname, filter_json=None, projection_json=None, sort_json={}, skip=0, limit=0):
            """
            查询匹配到的第一条数据
            :param tbname: MongoDB表名
            :param filter_json: 过滤json, _id必须用ObjectId("5ff2c39364efd65eed78597e")方式
            :param projection_json: 显示json
            :param sort_json: 排序json
            :param skip: 跳过行数
            :param limit: 限制行数
            :return:
            """
            tb = eval("self.db_client.{}".format(tbname))
            result = tb.find_one(filter=filter_json, projection=projection_json, sort=list(sort_json.items()),
                                 skip=skip,
                                 limit=limit)
            if result is not None:
                if result.__contains__("_id"):
                    result['_id'] = str(result['_id'])
            return result

        def find(self, tbname, filter_json=None, projection_json=None, sort_json={}, skip=0, limit=0):
            """
            查询多条数据
            :param tbname: MongoDB表名
            :param filter_json: 过滤json, _id必须用ObjectId("5ff2c39364efd65eed78597e")方式
            :param projection_json: 显示json
            :param sort_json: 排序json
            :param skip: 跳过行数
            :param limit: 限制行数
            :return:
            """
            result = []
            tb = eval("self.db_client.{}".format(tbname))
            items = tb.find(filter=filter_json, projection=projection_json, sort=list(sort_json.items()), skip=skip,
                            limit=limit)
            for item in items:
                if item.__contains__("_id"):
                    item['_id'] = str(item['_id'])
                result.append(item)
            return result


class RedisTools(object):
    """
    Redis数据库工具类
    """

    def __init__(self, host, port, dbname, password=None):
        """
         指定所处环境及数据库名称
         :param host: 主机地址
         :param port: 主机端口
         :param dbname: 数据库名称
         :param password: 密码
         """
        pool = redis.ConnectionPool(host=host, port=port, password=password, db=dbname, decode_responses=True)
        self.redisSession = redis.StrictRedis(connection_pool=pool, charset='utf-8')

    @property
    def redis(self):
        """
        获取redis实例
        :return:
        """
        return self.redisSession


class RedisSentinelTools(object):
    def __init__(self, service, password, instances):
        """
        Sentinel哨兵模式配置
        :param service: sentinel服务名称
        :param password: sentinel服务密码
        :param instances: sentinel实例，入参形式如[(host1,port1),(host2,port2)]
        """
        self._service = service
        self._password = password
        self._instances = instances
        self.sentinelSession = Sentinel(sentinels=self._instances, socket_timeout=0.5)

    @property
    def sentinel(self):
        """
        获取sentinel实例
        :return:
        """
        return self.sentinelSession

    def discover_master(self, service_name=None):
        """
        获取Redis主服务器地址
        :param service_name: 主服务名称
        :return:
        """
        return self.sentinelSession.discover_master(
            service_name=self._service if service_name is None else service_name)

    def discover_slaves(self, service_name=None):
        """
        获取Redis从服务器地址
        :param service_name：主服务名称
        :return:
        """
        return self.sentinelSession.discover_slaves(
            service_name=self._service if service_name is None else service_name)

    def master_instance(self, dbname, service_name=None, password=None):
        """
        获取主服务器实例，用于操作redis
        :param dbname: 数据库名称
        :param service_name: 主服务名称
        :param password: 主服务密码
        :return:
        """
        return self.sentinelSession.master_for(service_name=self._service if service_name is None else service_name,
                                               password=self._password if password is None else password,
                                               db=dbname)

    def slave_instance(self, dbname, service_name=None, password=None):
        """
        获取从服务器实例，用于操作redis
        :param dbname: 数据库名称
        :param service_name: 主服务名称
        :param password: 主服务密码
        :return:
        """
        return self.sentinelSession.slave_for(service_name=self._service if service_name is None else service_name,
                                              password=self._password if password is None else password,
                                              db=dbname)
